PRO pfold, time,rate,profile,period=period,nbins=nbins,time0=time0, $
           proferr=proferr,gap=gap,nogap=nogap,tolerance=tolerance, $
           phbin=phbin,ttot=ttot, raterr=raterr,chatty=chatty,npts=npts, $
           quantiles=quantiles, statquant=quant,dt=dt,gti=gti
;+
; NAME:
;       pfold
;
;
; PURPOSE:
;       folds a lightcurve with a given period and returns the resulting
;       profile
;
;
; CATEGORY:
;       lightcurve
;
;
; CALLING SEQUENCE:
;       pfold,time,rate,profile,period=period,nbins=nbins,time0=time0,
;             nbins=nbins,proferr=proferr,phbin=phbin,ttot=ttot, 
;             quantiles=quantiles,quant=quant,dt=dt
; 
; INPUTS:
;       time    : the starting time of each rate bin
;       rate    : the count rate of each bin (if rate is omitted, the
;                 code assumes EVENT data instead, and time is the
;                 time at which each event was detected
;
;
; OPTIONAL INPUTS:
;       nbins   : the number of phase bins to use
;       time0   : use time0 instead of the first entry in the
;                 timecolumn as zerotime of phase bin
;       gti     : an array of good time intervals
;                     gti(0,*) contains the starttimes,
;                     gti(1,*) contains the stoptimes.
;                 if given, the exposure time per phase bin (ttot) is
;                 computed from this information. alternatively, for
;                 lightcurves, the gap detection can be used.
;       raterr  : uncertainty of the counting rates in rate(*)
;       dt      : width of each time bin   
;       gap     : An array containing the indices of gaps in the
;                 lightcurve (i.e. time(gap(2)) is the starting time
;                 of the 3rd gap in the lightcurve). If gap is not
;                 given and nogap is not set, the code determines gap
;                 using the subroutine timegap. only used if gti is unknown.
;      tolerance: parameter defining the lower limit for the gap
;                 length; the reference is the time difference
;                 between the first and second entry in the time
;                 array; tolerance defines the maximum allowed relative
;                 deviation from this reference bin length; 
;                 default: 1e-8; this parameter is passed to timegap
;                     (see timgap.pro for further explanation)
;
;	
; KEYWORD PARAMETERS:
;       nogap   : If set, the lightcurve is assumed to NOT contain any
;                 bins
;       quantiles: If set, return the 1st,2nd, and 3rd quantile in 
;                 keyword statquant (only for lightcurves)
;
; OUTPUTS:
;       profile : the pulse profile (counts/sec)
;
; OPTIONAL OUTPUTS:
;       proferr : Array containing the statistical uncertainty of each
;                 bin of the profile. computed from raterr for
;                 lightcurves, for event data, Poissonian errors are
;                 assumed.
;       phbin   : array containing the start-phase of each bin of the
;                 profile
;       ttot    : array containing the total observing time
;                 accumulated in each bin of the profile
;       npts    : total number of rate points in each bin of the
;                 profile, or total number of events in each bin.
;       statquant: a 2d array, quant(0,*) contains the 1st, quant(1,*)
;                 the 2nd, and quant(2,*) the 3rd quantile for each
;                 pulse point; only for lightcurve data
;
; RESTRICTIONS:
;
;
;
; PROCEDURE:
;
;
;
; EXAMPLE:
;       nbins=128
;       pfold,time,counts,profile,283.,nbins=nbins
;       phase=findgen(nbins)/(nbins-1)
;       plot,phase,profile
;
;
; MODIFICATION HISTORY:
; $Log: pfold.pro,v $
; Revision 1.7  2003/06/30 10:26:25  wilms
; complete rewrite (Ljuba/JW), now also allows event lists
;
;
;       Version 1.0: Derived from foldlc, Joern Wilms, 1998/05/14
;       Version 1.1: J.W., 1998/08/09, added quantile keyword
;       Version 2.0: J.W., 1999/08/04:
;          * now use the histogram function for determining the
;            countrates contributing to each phase bin
;            (speed improvement roughly a factor of 20%)
;          * compute width of time bins only if desired
;            (slight speed improvement)
;       CVS Version 1.2: JW/PR, 2001.07.20
;          code now produces exact results in case time-time0 is negative
;       CVS Version 1.3: MK, 2001.10.18
;          added keyword tolerance
;       Version 1.4: JW, 2002.02.12
;          repaired tolerance keyword. now calls with tolerance eq
;          0. are possible
;       Version 1.5: JW/SF, 2002.08.15
;          correction of code in the example
;       Version 1.6: JW: CHANGE TO CVS LOGGING
;-

  ;; two non-keyword arguments: we're working on event data
  ;; three non-keyword arguments: we're working on a light curve
  eventdata=0
  IF (n_params() EQ 2) THEN eventdata=1

  IF (n_params() LT 2 OR n_params() GT 3) THEN BEGIN 
      message,'syntax: pfold,time,profile for event data',/informational
      message,'        pfold,time,rate,profile for a light curve'
  ENDIF 

  IF (time[0] EQ 0. AND n_elements(time0) NE 0) THEN BEGIN 
      message,'time[0]=0 and you have set a time0; this will',/informational
      message,'most probably not give the result you expect.',/informational
  ENDIF 

   IF (n_elements(time0) EQ 0) THEN time0=time[0]
   IF (n_elements(nbins) EQ 0) THEN nbins=20

   ;; event data: no gti file known
   IF (eventdata EQ 1 AND n_elements(gti) EQ 0) THEN BEGIN 
       message,'eventdata: GTI not known, using time of first',/informational
       message,'  and last event. This is most probably NOT',/informational
       message,'  what you want!',/informational
       gti=fltarr(2,1)
       gti[0,0]=time[0]
       gti[1,0]=time[n_elements(time)-1]
   ENDIF 

;   ;; correct for a weird oddity in the event data code below
;   ;; (should be repaired, but we don't have the time for this
;   ;; at the moment)
;   IF (eventdata EQ 1 AND time0 GT time[0]) THEN BEGIN
;       message,'eventdata: time0 must be <= begin of light curve'
;   ENDIF 

   IF (eventdata EQ 1 AND keyword_set(quantiles)) THEN BEGIN 
       message,'quantiles can only be computed for lightcurves'
   ENDIF 

   ;; Set gap tolerance to default value (only for lightcurves)
   ttol=1e-8 
   IF (n_elements(tolerance) NE 0) THEN ttol=tolerance

   ;; disable chatty if not set
   IF (n_elements(chatty) EQ 0) THEN chatty=0

   ;; Width of each time-bin, take care of wrap-around
   IF (n_elements(dt) EQ 0) THEN BEGIN 
       dt=temporary(shift(time,-1))-time
       dt(n_elements(time)-1)=dt(0)
   ENDIF 

   ;;
   ;; start of phase bins, including phase 1 (is removed at end of
   ;; code and makes code more elegant)
   ;;
   phbin=dindgen(nbins+1)/nbins
   size_bin=period/nbins ;; width of each time bin

   ;;
   ;; If the gti file on which the lightcurve or event data are based
   ;; on is known, we can compute the exact exposure for each bin
   ;; (if not:  compute ttot from the bin size)
   ;;
   ttot=dblarr(nbins)
   IF (n_elements(gti) NE 0)   THEN BEGIN 
       ;;
       ;; Loop over all good time intervals
       ;;
       FOR i=0, n_elements(gti[0,*])-1 DO BEGIN

           ;;
           ;; compute information for start and stop times
           ;;
           starttime=gti[0,i]
           stoptime=gti[1,i]

           ;; period number of start period
            nstart = long((starttime - time0)/period) 
            IF (starttime LT time0) THEN nstart = nstart - 1d

           ;; period number of stop period
            nstop  = long((stoptime  - time0)/period)
            IF (stoptime LT time0) THEN nstop   = nstop - 1d
      
           phasestart = ((starttime-time0) MOD  period)/period
           IF (phasestart LT 0.) THEN phasestart = phasestart+ 1.

           ;; phase of stop period            
           phasestop  = ((stoptime-time0)  MOD  period)/period
           IF (phasestop  LT 0.) THEN phasestop  = phasestop + 1.

           ;; nbstart -- number of start bin 
           ;; nbstop  -- nubber of stop bin
           
           nbstart = long(phasestart*nbins)
           nbstop  = long(phasestop*nbins)
           
           IF (nstart NE nstop) THEN BEGIN
               ;;
               ;; Case that the start and stop time of the GTI do 
               ;; NOT fall into the same period bin
               ;;

               ;;
               ;; Take care of the periods between
               ;; the start and the stop interval
               ;;
               ttot[*] = ttot[*] + (nstop - (nstart+1))*size_bin

               ;;
               ;; Take care of the first and the last period interval
               ;;

               ;; ... first period interval
               ttot[nbstart]=ttot[nbstart] + $
                 (phbin[nbstart+1] - phasestart)*period
               IF (nbstart LT nbins-1) THEN BEGIN 
                   ttot[nbstart+1:nbins-1]=ttot[nbstart+1:nbins-1]+size_bin
               ENDIF 

               ;; ... last period interval
               IF (nbstop NE 0) THEN BEGIN 
                   ttot[0:nbstop-1] = ttot[0:nbstop-1]+size_bin
               ENDIF 
               ttot[nbstop]=ttot[nbstop] + $
                 (phasestop - phbin[nbstop])*period

           ENDIF ELSE BEGIN 
               ;;
               ;; GTI is shorter than one period
               ;; and falls into the same period interval
               ;;

               IF (nbstart EQ nbstop) THEN BEGIN 
                   ;;
                   ;; GTI only falls into ONE phase bin
                   ;;
                   ttot[nbstart]=ttot[nbstart]+(phasestop-phasestart)*period
               ENDIF ELSE BEGIN 
                   ;;
                   ;; GTI falls into more than one phase bin
                   ;;


                   IF (nbstart+1 NE nbstop) THEN BEGIN 
                       ;; ... phase bins between start and stop bin
                                      
                       ttot[nbstart+1:nbstop-1] = $
                         ttot[nbstart+1:nbstop-1] + size_bin
                   ENDIF 
                   ;; ... start and stop bin
                   ttot[nbstart]=ttot[nbstart] + $
                     (phbin[nbstart+1] - phasestart)*period
                   ttot[nbstop]=ttot[nbstop] + $
                     (phasestop - phbin[nbstop])*period
               ENDELSE 
           ENDELSE 
       ENDFOR  
   ENDIF 

   ;; ... computation of the pulse profile
   
   ;;
   ;; Compute phase of each timebin
   ;;
   phase = ((time-time0) MOD period)/period

   ;; take care of case that there could be times with time < time0
   ndx=where(phase LT 0.,nummin)
   IF (nummin NE 0) THEN phase[ndx]=phase[ndx]+1.

   ;;
   ;; If lc  contains gaps: mark them invalid
   ;; (should we take into account the GTI information here??)
   ;;
   IF (eventdata EQ 0) THEN BEGIN 
       IF (NOT keyword_set(nogap)) THEN BEGIN 
           IF (n_elements(gap) EQ 0) THEN BEGIN 
               timegap,time,gap,dblock,tolerance=ttol,chatty=chatty
           ENDIF 
           IF (gap(0) NE -1) THEN phase(gap)=2.
       ENDIF 
   END 

   profile=fltarr(nbins)
   proferr=fltarr(nbins)
   npts=lonarr(nbins)

   ;;
   ;; Get indices to all times falling into each phase bin
   ;;
   dummy=histogram(phase,binsize=1./nbins,min=0.,max=1., $
                   reverse_indices=r)
   ;;
   ;; Compute the profiles
   ;;
   IF (eventdata EQ 1) THEN BEGIN 
       ;;
       ;; Event data
       ;;
       FOR i=0,nbins-1 DO BEGIN 
           IF (r[i] EQ r[i+1]) THEN BEGIN 
               IF (keyword_set(chatty)) THEN BEGIN 
                   print, 'Warning: phase '+strtrim(string(i),2)+ $
                     ' contains no bins'
               ENDIF 
               npts[i]=0L ;; not really needed...
           ENDIF ELSE BEGIN
               npts[i]=n_elements(r[r[i]:r[i+1]-1])
           ENDELSE 
       ENDFOR 

       ndx=where(ttot NE 0.)
       profile=npts[ndx]/ttot[ndx]
       proferr=sqrt(npts[ndx])/ttot[ndx]

   ENDIF ELSE BEGIN 

       ;;
       ;; Lightcurve data
       ;;

       IF (keyword_set(quantiles)) THEN BEGIN 
           quant=fltarr(3,nbins)
       ENDIF 

       FOR i=0,nbins-1 DO BEGIN 
           IF (r[i] EQ r[i+1]) THEN BEGIN 
               IF (keyword_set(chatty)) THEN BEGIN 
                   print, 'Warning: phase '+strtrim(string(i),2)+ $
                     ' contains no bins'
               ENDIF 
               profile[i]=0.
               npts[i]=0
               IF (n_elements(gti) EQ 0) THEN ttot(i)=0.
           END ELSE BEGIN
               ndx=r[r[i]:r[i+1]-1]
               dte=dt[ndx]
               IF (n_elements(gti) EQ 0) THEN ttot[i]=total(dte)
               npts[i]=n_elements(ndx)
               ;; need dt because of boundaries and GTI's
               profile[i]=total(rate[ndx]*dte)/ttot[i]
           
               IF (n_elements(raterr) NE 0) THEN BEGIN 
                   proferr[i]=sqrt(total((raterr[ndx]*dte)^2.))/ttot[i]
               ENDIF 
           
               ;; compute statistical quantiles if desired
               IF (keyword_set(quantiles)) THEN BEGIN 
                   dist=rate[ndx]
                   nnn=sort(dist)
                   n25=fix(npts[i]*0.25)
                   n50=fix(npts[i]*0.50)
                   n75=fix(npts[i]*0.75)
                   quant(0,i)=dist[nnn[n25]]
                   quant(1,i)=dist[nnn[n50]]
                   quant(2,i)=dist[nnn[n75]]
               ENDIF 
           END
       ENDFOR 
   ENDELSE 

   ;; truncate last phase bin
   phbin=phbin[0:nbins-1]

   ;; event data: return profile in the 2nd argument
   IF (eventdata EQ 1) THEN rate=profile
END 




